"""High level API for email."""
from typing import Any, Dict, List, Optional
from uuid import UUID

from typist import SecretaryClient

from .message import Message, MessageEnvelope


class Email(Message):
    """Provide a structure for Email message."""

    rendered_fields = ('subject', 'body', 'body_html')

    def __init__(self,
                 recipient: str,
                 subject_template: str,
                 body_template: str,
                 *,
                 body_template_html: str = None,
                 subject_template_uuid: UUID = None,
                 body_template_uuid: UUID = None,
                 body_template_html_uuid: UUID = None,
                 sender: str = None,
                 extra_headers: Dict[str, str] = None,
                 context: Dict[str, Any] = None,
                 attachments: List[str] = None,
                 type: str = None,
                 secretary_client: SecretaryClient = None):
        """Create Email message."""
        super().__init__(type=type, secretary_client=secretary_client)
        self.sender = sender
        self.recipient = recipient
        self.extra_headers = extra_headers or {}
        self.subject_template = subject_template
        self.body_template = body_template
        self.body_template_html = body_template_html
        self.subject_template_uuid = subject_template_uuid
        self.body_template_uuid = body_template_uuid
        self.body_template_html_uuid = body_template_html_uuid
        self.context = context or {}
        self.attachments = attachments or []

    @property
    def subject(self) -> str:
        """Render subject using django-secretary."""
        return self._secretary_render('subject', template=self.subject_template_uuid or self.subject_template)

    @property
    def body(self) -> str:
        """Render body using django-secretary."""
        return self._secretary_render('body', template=self.body_template_uuid or self.body_template)

    @property
    def body_html(self) -> Optional[str]:
        """Render body_html using django-secretary."""
        template = self.body_template_html_uuid or self.body_template_html
        if template:
            return self._secretary_render(
                'body_html',
                template=template,
                render_method_name='render_html',
            )
        else:
            return None

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with e-mail info.

        This method is useful for email serialization.
        """
        info = super().to_dict()
        info.update({
            'sender': self.sender,
            'recipient': self.recipient,
            'extra_headers': self.extra_headers,
            'subject_template': self.subject_template,
            'body_template': self.body_template,
            'body_template_html': self.body_template_html,
            'subject_template_uuid': self.subject_template_uuid,
            'body_template_uuid': self.body_template_uuid,
            'body_template_html_uuid': self.body_template_html_uuid,
            'context': self.context,
            'attachments': self.attachments,
        })
        return info


class EmailEnvelope(MessageEnvelope[Email]):
    """Provide a structure for Email envelope."""

    message_cls = Email

    def __init__(self, message: Email, *, message_id: str = None, **kwargs):
        super().__init__(message=message, **kwargs)
        self.message_id = message_id

    @classmethod
    def from_dict(cls, data: Dict[str, Any], *, secretary_client: SecretaryClient = None) -> 'EmailEnvelope':
        """Create instance from a dictionary.

        Raises:
            ValueError: If required keys for Email creation are missing
        """
        email_envelope = super().from_dict(data, secretary_client=secretary_client)
        email_envelope.message_id = data.get('message_id')
        return email_envelope

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with e-mail envelope info.

        This method is useful for email envelope serialization.
        """
        info = super().to_dict()
        info.update({'message_id': self.message_id})
        return info
